// file:CLogger.cxx
/** @file
 * @brief cfpロガー・クラス・cxxファイル
 * 
 * cfpロッガー・クラスの実装
 * @author yanch
 * @date 2006-10-21 作成
 * @date 2006-11-19 propファイルのパラメタ区切りにスペースが無い場合に対応
 * @date 2006-11-19 propファイルのパラメタ区切り文字にタブを使える様に対応
 * @date 2007-11-16 namespace に対応
 * @date 2007-11-16 unicode に対応
 * @date 2007-11-16 ソースファイルの拡張子をgccにならい、「.cxx」とした。
 * @date 2007-11-16 ヘッダファイルの拡張子を「.hxx」とした。
 * $Revision: 1.004$
 */
#define STRICT
#include <clocale>
#include <cstdio>
#include <cstring>
#include <ctime>
#include <cwchar>
#include <string>
#include <cfp/logger/sub/CLoggerProfile.hxx>
#include <cfp/logger/sub/CLoggerStringUtils.hxx>
#include <cfp/logger/CLogger.hxx>

static void fputsMessage(std::wstring strMessage);

/**
 * スタティック変数プロファイルの初期化。
 */
static cfp::logger::sub::CLoggerProfile s_profile = 
		cfp::logger::sub::CLoggerProfile();

/**
 * コンストラクタ。
 */
cfp::logger::CLogger::CLogger()
{
	// ロケールを日本語に設定。
	::_wsetlocale(LC_ALL, L"japanese");
}

/**
 * コンストラクタ。
 * 
 * @param[in] strPackageName パッケージ名
 */
cfp::logger::CLogger::CLogger(std::wstring strPackageName)
{
	// ロケールを日本語に設定。
	::_wsetlocale(LC_ALL, L"japanese");
	
	m_strPackageName = strPackageName;
}

/**
 * デストラクタ。
 */
cfp::logger::CLogger::~CLogger()
{
}

/**
 * デバック用・ログを出力。
 * 
 * @param[in] strMessage メッセージ
 */
void cfp::logger::CLogger::debug(std::wstring strMessage)
{
	std::wstring strBuffer = L"";
	if (s_profile.isDebug() == false)
	{
		return;
	}
	if (!strMessage.empty())
	{
		strBuffer = getMessage(L"Debug", strMessage);
		if (s_profile.isAppenderStdout())
		{
			::fputws(strBuffer.c_str(), stdout);
		}
		if (s_profile.isAppenderFile())
		{
			fputsMessage(strBuffer);
		}
	}
}

/**
 * エラー用・ログを出力。
 * 
 * @param[in] strMessage メッセージ
 */
void cfp::logger::CLogger::error(std::wstring strMessage)
{
	std::wstring strBuffer = L"";
	if (s_profile.isError() == false)
	{
		return;
	}
	if (!strMessage.empty())
	{
		strBuffer = getMessage(L"Error", strMessage);
		if (s_profile.isAppenderStdout())
		{
			::fputws(strBuffer.c_str(), stdout);
		}
		if (s_profile.isAppenderFile())
		{
			fputsMessage(strBuffer);
		}
	}
}

/**
 * ロッガーのインスタンスを取得。
 * 
 * @param[in] strPackageName パッケージ名
 * 
 * @return CLoggerのインスタンス
 */
cfp::logger::CLogger cfp::logger::CLogger::getLogger(
		std::wstring strPackageName)
{
	CLogger logger = CLogger::CLogger(strPackageName);
	
	return logger;
}

/**
 * ログ出力用のメッセージを組み立てる。
 * 
 * @param[in] strPrefix 出力用メッセージの前置詞
 * @param[in] strMessage メッセージ
 * 
 * @return 組み立てられたメッセージ
 */
std::wstring cfp::logger::CLogger::getMessage(std::wstring strPrefix
		, std::wstring strMessage)
{
	std::wstring strRet = L"";
	time_t ltime;
	struct tm *today;
	wchar_t tmpbuf[32] = L"";
	
	::time(&ltime);
	today = ::localtime(&ltime);
	if (today != NULL)
	{
		// YYYY-mm-dd.HH.MM.SS
		::wcsftime(tmpbuf, 31, L"%Y-%m-%d.%H:%M:%S", today);
		strRet += std::wstring(tmpbuf) + L" - ";
	}
	if (strPrefix.empty() == false)
	{
		strRet += strPrefix + L" - ";
	}
	strRet += m_strPackageName + L" - ";
	strRet += strMessage;
	
	return strRet;
}

/**
 * デバッグログ出力用プロパティを得る。
 * 
 * true:デバッグログ出力は有効、false:デバッグログ出力は無効。
 * 
 * @return debug
 */
bool cfp::logger::CLogger::isDebug()
{
	return s_profile.isDebug();;
}

/**
 * エラーログ出力用プロパティを得る。
 * 
 * true:エラーログ出力は有効、false:エラーログ出力は無効。
 * 
 * @return error
 */
bool cfp::logger::CLogger::isError()
{
	return s_profile.isError();
}

/**
 * トレースログ出力用プロパティを得る。
 * 
 * true:トレースログ出力は有効、false:トレースログ出力は無効。
 * 
 * @return trace
 */
bool cfp::logger::CLogger::isTrace()
{
	return s_profile.isTrace();
}

/**
 * ワーニングログ出力用プロパティを得る。
 * 
 * true:ワーニングログ出力は有効、false:ワーニングログ出力は無効。
 * 
 * @return warn
 */
bool cfp::logger::CLogger::isWarn()
{
	return s_profile.isWarn();
}

/**
 * トレース用・ログを出力。
 * 
 * @param[in] strMessage メッセージ
 */
void cfp::logger::CLogger::trace(std::wstring strMessage)
{
	std::wstring strBuffer = L"";
	if (s_profile.isTrace() == false)
	{
		return;
	}
	if (!strMessage.empty())
	{
		strBuffer = getMessage(L"Trace", strMessage);
		if (s_profile.isAppenderStdout())
		{
			::fputws(strBuffer.c_str(), stdout);
		}
		if (s_profile.isAppenderFile())
		{
			fputsMessage(strBuffer);
		}
	}
}

/**
 * ワーニング用・ログを出力。
 * 
 * @param[in] strMessage メッセージ
 */
void cfp::logger::CLogger::warn(std::wstring strMessage)
{
	std::wstring strBuffer = L"";
	if (s_profile.isWarn() == false)
	{
		return;
	}
	if (!strMessage.empty())
	{
		strBuffer = getMessage(L"Warn", strMessage);
		if (s_profile.isAppenderStdout())
		{
			::fputws(strBuffer.c_str(), stdout);
		}
		if (s_profile.isAppenderFile())
		{
			fputsMessage(strBuffer);
		}
	}
}

/**
 * ファイルにログを出力。
 * 
 * @param[in] strMessage メッセージ
 */
void fputsMessage(std::wstring strMessage)
{
	unsigned int iPos = 0;
	time_t ltime;
	FILE *pFile;
	std::wstring strA = L"";
	std::wstring strB = L"";
	std::wstring strC = L"";
	std::wstring strFileName = L"";
	wchar_t tmpbuf[16] = L"";
	struct tm *today;
	wchar_t wcharBuffer[1024];
	char charBuffer[1024];
	int iResult = 0;
	
	strFileName = s_profile.getAppenderFileName();
	iPos = strFileName.find_first_of(L".");
	if (iPos != std::wstring::npos)
	{
		strA = strFileName.substr(0, iPos);
		strC = strFileName.substr(iPos);
		::time(&ltime);
		today = ::localtime(&ltime);
		if (today != NULL)
		{
			// YYYYmmdd
			::wcsftime(tmpbuf, 15, L"%Y%m%d", today);
			strB = std::wstring(tmpbuf);
		}
		strFileName = strA + strB + strC;
	}
	pFile = ::_wfopen(strFileName.c_str(), L"a");
	if (pFile != NULL)
	{
		// utf16 to utf8
		::memset(wcharBuffer, 0, sizeof(wcharBuffer));
		::memset(charBuffer, 0, sizeof(charBuffer));
		::wcsncpy(wcharBuffer, strMessage.c_str()
				, (sizeof(wcharBuffer) / sizeof(wchar_t) - 1));
		iResult = cfp::logger::sub::CLoggerStringUtils::convertUtf16ToUtf8(
				charBuffer, sizeof(charBuffer) - 1, wcharBuffer
				, ::wcslen(wcharBuffer));
		if (iResult != false)
		{
			::fputs(charBuffer, pFile);
		}
		::fclose(pFile);
	}
}
