// file:CLoggerProfile.cxx
/** @file
 * @brief cfpロガー・プロファイル・クラス・cxxファイル
 * 
 * cfpロッガー・プロファイル・クラスの実装
 * @author yanch
 * @date 2006-10-21 作成
 * @date 2006-11-19 propファイルのパラメタ区切りにスペースが無い場合に対応
 * @date 2006-11-19 propファイルのパラメタ区切り文字にタブを使える様に対応
 * @date 2007-11-16 namespace に対応
 * @date 2007-11-16 unicode に対応
 * @date 2007-11-16 ソースファイルの拡張子をgccにならい、「.cxx」とした。
 * @date 2007-11-16 ヘッダファイルの拡張子を「.hxx」とした。
 * $Revision: 1.004$
 */
#define STRICT
#include <sys/types.h>
#include <sys/stat.h>
#include <cstdio>
#include <cstring>
#include <string>
#include <cfp/logger/sub/CLoggerPropertyUtils.hxx>
#include <cfp/logger/sub/CLoggerProfile.hxx>

/**
 * コンストラクタ。
 */
cfp::logger::sub::CLoggerProfile::CLoggerProfile()
{
	// ロケールを日本語に設定。
	::_wsetlocale(LC_ALL, L"japanese");
	
	m_strPropertyFileName = L"cfp_logger.prop";
	loadProfile();
}

/**
 * デストラクタ。
 */
cfp::logger::sub::CLoggerProfile::~CLoggerProfile()
{
}

/**
 * ログ出力ファイル名用プロパティを得る。
 * 
 * @return appenderFileName
 */
std::wstring cfp::logger::sub::CLoggerProfile::getAppenderFileName()
{
	if (isUpdateProfile())
	{
		loadProfile();
	}
	return m_strAppenderFileName;
}

/**
 * ログ出力ファイル用プロパティを得る。
 * 
 * true:ログファイル出力有効、false:ログファイル出力無効。
 * 
 * @return appenderFile
 */
bool cfp::logger::sub::CLoggerProfile::isAppenderFile()
{
	if (isUpdateProfile())
	{
		loadProfile();
	}
	return m_boAppenderFile;
}

/**
 * 標準出力ログ用プロパティを得る。
 * 
 * true:標準出力ログ有効、false:標準出力ログ無効。
 * 
 * @return appenderStdout
 */
bool cfp::logger::sub::CLoggerProfile::isAppenderStdout()
{
	if (isUpdateProfile())
	{
		loadProfile();
	}
	return m_boAppenderStdout;
}

/**
 * デバックログ出力用プロパティを得る。
 * 
 * true:デバッグログ出力は有効、false:デバッグログ出力は無効。
 * 
 * @return debug
 */
bool cfp::logger::sub::CLoggerProfile::isDebug()
{
	if (isUpdateProfile())
	{
		loadProfile();
	}
	return m_boDebug;
}

/**
 * エラーログ出力用プロパティを得る。
 * 
 * true:エラーログ出力は有効、false:エラーログ出力は無効。
 * 
 * @return error
 */
bool cfp::logger::sub::CLoggerProfile::isError()
{
	if (isUpdateProfile())
	{
		loadProfile();
	}
	return m_boError;
}

/**
 * トレースログ出力用プロパティを得る。
 * 
 * true:トレースログ出力は有効、false:トレースログ出力は無効。
 * 
 * @return trace
 */
bool cfp::logger::sub::CLoggerProfile::isTrace()
{
	if (isUpdateProfile())
	{
		loadProfile();
	}
	return m_boTrace;
}

/**
 * プロファイルが更新されているかどうか調べる。
 * 
 * true:プロパティファイルは更新されている、
 * false:プロパティファイルは更新されていない。
 * 
 * プロファイルの更新フラグは、loadProfile()実行時に、falseと設定される。
 * その後に、プロファイルが更新されていれば、
 * プロファイル更新フラグはtrueとして、処理されます。
 * 
 * @return プロファイル更新状態
 */
bool cfp::logger::sub::CLoggerProfile::isUpdateProfile()
{
	struct _stat statBuf;
	int iResult;
	bool boRet = false;
	
	memset(&statBuf, 0, sizeof(statBuf));
	iResult = ::_wstat(m_strPropertyFileName.c_str(), &statBuf);
	if (iResult != 0)
	{
		std::wstring wstrMessage;
		wstrMessage = L"cfp_logger system error - ファイルの情報を取得出来ませんでした。{";
		wstrMessage += m_strPropertyFileName;
		wstrMessage += L"}\n";
		::fwprintf(stderr, L"%s", wstrMessage.c_str());
	}
	else
	{
		if (m_timeMTime < statBuf.st_mtime)
		{
			boRet = true;
		}
	}
	return boRet;
}

/**
 * ワーニングログ出力用プロパティを得る。
 * 
 * true:ワーニングログ出力は有効、false:ワーニングログ出力は無効。
 * 
 * @return warn
 */
bool cfp::logger::sub::CLoggerProfile::isWarn()
{
	if (isUpdateProfile())
	{
		loadProfile();
	}
	return m_boWarn;
}

/**
 * プロファイルを読み込む。
 */
void cfp::logger::sub::CLoggerProfile::loadProfile()
{
	cfp::logger::sub::CLoggerPropertyUtils propertyUtils
			= cfp::logger::sub::CLoggerPropertyUtils::getLoggerPropertyUtils(
			m_strPropertyFileName);
	std::wstring strTrue = L"true";
	std::wstring strFalse = L"false";
	std::wstring strValue = L"";
	
	// mtime
	struct _stat statBuf;
	int iResult;
	
	::memset(&statBuf, 0, sizeof(statBuf));
	iResult = ::_wstat(m_strPropertyFileName.c_str(), &statBuf);
	if (iResult != 0)
	{
		std::wstring wstrMessage;
		wstrMessage = L"cfp_logger system error - ファイルの情報を取得出来ませんでした。{";
		wstrMessage += m_strPropertyFileName;
		wstrMessage += L"}\n";
		::fwprintf(stderr, L"%s", wstrMessage.c_str());
	}
	else
	{
		m_timeMTime = statBuf.st_mtime;
	}
	
	// DEBUG
	strValue = propertyUtils.getPropertyValue(L"DEBUG");
	if (strTrue.compare(strValue) == 0)
	{
		m_boDebug = true;
	}
	else
	{
		m_boDebug = false;
	}
	
	// ERROR
	strValue = propertyUtils.getPropertyValue(L"ERROR");
	if (strTrue.compare(strValue) == 0)
	{
		m_boError = true;
	}
	else
	{
		m_boError = false;
	}
	
	// TRACE
	strValue = propertyUtils.getPropertyValue(L"TRACE");
	if (strTrue.compare(strValue) == 0)
	{
		m_boTrace = true;
	}
	else
	{
		m_boTrace = false;
	}
	
	// WARN
	strValue = propertyUtils.getPropertyValue(L"WARN");
	if (strTrue.compare(strValue) == 0)
	{
		m_boWarn = true;
	}
	else
	{
		m_boWarn = false;
	}
	
	// APPENDER_FILENAME
	strValue = propertyUtils.getPropertyValue(L"APPENDER_FILENAME");
	if (strValue.empty())
	{
		m_strAppenderFileName = L"";
		m_boAppenderFile = false;
	}
	else
	{
		m_strAppenderFileName = strValue;
		m_boAppenderFile = true;
	}
	
	// APPENDER_STDOUT
	strValue = propertyUtils.getPropertyValue(L"APPENDER_STDOUT");
	if (strTrue.compare(strValue) == 0)
	{
		m_boAppenderStdout = true;
	}
	else
	{
		m_boAppenderStdout = false;
	}
}
