// file:CLoggerPropertyUtils.cxx
/** @file
 * @brief プロパティ・ユーティル・cxxファイル
 * 
 * プロパティ・ユーティルの実装。
 * @author yanch
 * @date 2006-10-21 作成
 * @date 2006-11-19 propファイルのパラメタ区切りにスペースが無い場合に対応
 * @date 2006-11-19 propファイルのパラメタ区切り文字にタブを使える様に対応
 * @date 2007-11-16 namespace に対応
 * @date 2007-11-16 unicode に対応
 * @date 2007-11-16 ソースファイルの拡張子をgccにならい、「.cxx」とした。
 * @date 2007-11-16 ヘッダファイルの拡張子を「.hxx」とした。
 * $Revision: 1.004$
 */
#define STRICT
#include <cstdio>
#include <cstring>
#include <cwchar>
#include <map>
#include <sstream>
#include <string>
#include <vector>
#include <cfp/logger/sub/CLoggerStringUtils.hxx>
#include <cfp/logger/sub/CLoggerPropertyUtils.hxx>

static std::vector<std::wstring> split(std::wstring strList
		, std::wstring strFileName);
static std::wstring trim(std::wstring strSrc);

/* 現在の処理中cfp_logger.propの行番号 */
static int s_numberOfLine = 0;

/**
 * コンストラクタ。
 */
cfp::logger::sub::CLoggerPropertyUtils::CLoggerPropertyUtils()
{
}

/**
 * デストラクタ。
 */
cfp::logger::sub::CLoggerPropertyUtils::~CLoggerPropertyUtils()
{
}

/**
 * ファイル名を取得。
 * 
 * @return ファイル名
 */
std::wstring cfp::logger::sub::CLoggerPropertyUtils::getFileName()
{
	return m_strFileName;
}

/**
 * CLoggerPropertyUtilクラスの実体を戻す。
 * 
 * @param[in] strFileName プロパティファイル名
 * @return CLoggerPropertyUtilsクラスの実体。
 */
cfp::logger::sub::CLoggerPropertyUtils cfp::logger::sub
		::CLoggerPropertyUtils::getLoggerPropertyUtils(
		std::wstring strFileName)
{
	CLoggerPropertyUtils utils;
	utils.m_strFileName = strFileName;
	
	return utils;
}

/**
 * プロパティ値を得る。
 * 
 * プロパティ・ファイルより、指定されたキーを用いて、値を取得する。
 * 失敗時には、空文字列を戻す。
 * 
 * @param[in] strKey プロパティ・キー
 * @return value プロパティ値
 */
std::wstring cfp::logger::sub::CLoggerPropertyUtils::getPropertyValue(
		std::wstring strKey)
{
	if (strKey.empty())
	{
		return L"";
	}
	loadProperties();
	std::map<std::wstring, std::wstring>::iterator it;
	it = m_mapProperties.find(strKey);
	if (it == m_mapProperties.end())
	{
		// キーが見つかりません。
		return L"";
	}
	
	return ((*it).second);
}

/**
 * プロパティ・ファイルを読み込み。
 * 
 * プロパティ・ファイルよりプロパティ・キーと、プロパティ値のセットを
 * 読み込みます。
 * 失敗時には、stderrエラーメッセージを出力する。
 */
void cfp::logger::sub::CLoggerPropertyUtils::loadProperties()
{
	FILE *pFile = NULL;
	char charBuffer[1024];
	wchar_t wcharBuffer[1024];
	int iResult = 0;
	
	// 行番号を0クリア
	s_numberOfLine = 0;
	
	// マップをクリア。
	m_mapProperties.clear();
	pFile = ::_wfopen(m_strFileName.c_str(), L"r");
	do
	{
		if (pFile == NULL)
		{
			std::wstring wstrMessage;
			wstrMessage = L"cfp_logger system error - ファイルが見つかりません。{";
			wstrMessage += m_strFileName;
			wstrMessage += L"}\n";
			::fwprintf(stderr, L"%s", wstrMessage.c_str());
			break;
		}
		
		while (NULL != ::fgets(charBuffer, sizeof(charBuffer), pFile))
		{
			// 行番号を進める。
			s_numberOfLine++;
			
			// utf8 to utf16
			::memset(wcharBuffer, 0, sizeof(wcharBuffer));
			iResult = cfp::logger::sub::CLoggerStringUtils
					::convertUtf8ToUtf16(wcharBuffer
					, sizeof(wcharBuffer) - 1, charBuffer
					, ::strlen(charBuffer));
			if (iResult == false)
			{
				continue;
			}
			
			std::vector<std::wstring> vStrWork5;
			std::wstring strWork1(wcharBuffer);
			strWork1 = strWork1.substr(0, strWork1.length() - 1);
			std::wstring strWork2 = trim(strWork1);
			vStrWork5 = split(strWork2, m_strFileName);
			if (vStrWork5.empty() == false)
			{
				std::wstring strKey, strValue;
				strKey = vStrWork5.at(0);
				strValue = vStrWork5.at(1);
				m_mapProperties.insert(
						std::map<std::wstring, std::wstring>::value_type(
						strKey, strValue));
			}
		}
		::fclose(pFile);
	} while (0);
}

/**
 * ファイル名を設定。
 * 
 * プロパティ・ファイル名を再設定する時に用いる。
 * 
 * @param[in] strFileName ファイル名
 */
void cfp::logger::sub::CLoggerPropertyUtils::setFileName(
		std::wstring strFileName)
{
	m_strFileName = strFileName;
}

/**
 * 一行の文字列を情報に分解。
 * 
 * 一行の文字列情報を、プロパティ・キー、プロパティ値に分解して戻す。
 * 戻り値のvectorには、at(0)にプロパティ・キーを、at(1)にプロパティ値を
 * 戻します。
 * プロパティ情報の解析に失敗した場合や、strListにプロパティ情報が含まれない
 * 場合には空のvectorを戻します。
 * 
 * @param[in] strList プロパティ情報を表す文字列
 * @param[in] strFileName ファイル名(エラーメッセージ用)
 * @return 戻り値
 */
std::vector<std::wstring> split(std::wstring strList, std::wstring strFileName)
{
	bool boValidValue;						// valueが妥当であるかと示すフラグ
	std::vector<std::wstring> vString;		// 戻り値作成用リスト
	std::wstring strDest = L"";				// 編集中文字列
	std::wstring strEqual = L"=";			// 等号
	std::wstring strKey = L"";				// キー文字列取得用
	std::wstring strPosChar = L"";			// カーソル位置文字
	std::wstring strPosWChar = L"";			// カーソル位置から２文字
	std::wstring strQuot = L"\"";			// 引用記号
	std::wstring strQuotQuot = L"\"\"";		// 引用記号x2
	std::wstring strSharp = L"#";			// シャープ記号
	std::wstring strSpace = L" ";			// 空白記号
	std::wstring strTab = L"	";			// タブ記号
	std::wstring strValue = L"";			// 値文字列取得用
	std::vector<std::wstring> vStringDummy;	// エラー時に戻すダミー
	do
	{
		// パラメタが空であるならば、終了。
		if (strList.empty())
		{
			break;
		}
		// 前後の空白をトリミング
		strDest = trim(strList);
		// トリミング後に空であるならば、終了。
		if (strDest.empty())
		{
			break;
		}
		// 最初の文字が#であるならば、終了。
		strPosChar = strDest.substr(0, 1);
		if (strSharp.compare(strPosChar) == 0)
		{
			break;
		}
		// 最初の文字が引用符であるならば、エラー終了。
		if (strQuot.compare(strPosChar) == 0)
		{
			std::wostringstream os;
			std::wstring wstrMessage;
			os << L"cfp_logger system error - キーが引用符で始まっています。(";
			os << L"File:" << strFileName << L" Line " << s_numberOfLine;
			os << L":)\n";
			wstrMessage = os.str();
			::fwprintf(stderr, L"%s", wstrMessage.c_str());
			break;
		}
		// 最初の文字が等号であるならば、エラー終了。
		if (strEqual.compare(strPosChar) == 0)
		{
			std::wostringstream os;
			std::wstring wstrMessage;
			os << L"cfp_logger system error - キーが等号で始まっています。(";
			os << L"File:" << strFileName << L" Line " << s_numberOfLine;
			os << L":)\n";
			wstrMessage = os.str();
			::fwprintf(stderr, L"%s", wstrMessage.c_str());
			break;
		}
		// key を取得
		strKey = L"";
		while (0 < strDest.length())
		{
			strPosChar = strDest.substr(0, 1);
			strDest = strDest.substr(1);
			if (strSpace.compare(strPosChar) == 0
					|| strTab.compare(strPosChar) == 0)
			{
				// posにある文字はspaceまたはtab
				break;
			}
			else if (strEqual.compare(strPosChar) == 0)
			{
				// posにある文字はequal
				strDest = strEqual + strDest;	// 文字列の前方にequalを付加
				break;
			}
			else if (strQuot.compare(strPosChar) == 0)
			{
				// posにある文字はquot
				std::wostringstream os;
				std::wstring wstrMessage;
				os << L"cfp_logger system error - キーに引用符が含まれています。(";
				os << L"File:" << strFileName << L" Line " << s_numberOfLine;
				os << L":)\n";
				wstrMessage = os.str();
				::fwprintf(stderr, L"%s", wstrMessage.c_str());
				return vStringDummy;
			}
			else if (strSharp.compare(strPosChar) == 0)
			{
				// posにある文字はsharp
				std::wostringstream os;
				std::wstring wstrMessage;
				os << L"cfp_logger system error - キーにシャープ記号が含まれています。(";
				os << L"File:" << strFileName << L" Line " << s_numberOfLine;
				os << L":)\n";
				wstrMessage = os.str();
				::fwprintf(stderr, L"%s", wstrMessage.c_str());
				return vStringDummy;
			}
			strKey += strPosChar;
		}
		// equal記号を取得
		strDest = trim(strDest);
		strPosChar = strDest.substr(0, 1);
		if (strEqual.compare(strPosChar) != 0)
		{
			// equal記号が見つからなかった。
			std::wostringstream os;
			std::wstring wstrMessage;
			os << L"cfp_logger system error - equal記号があるべき場所に見つかりません。(";
			os << L"File:" << strFileName << L" Line " << s_numberOfLine;
			os << L":)\n";
			wstrMessage = os.str();
			::fwprintf(stderr, L"%s", wstrMessage.c_str());
			return vStringDummy;
		}
		strDest = strDest.substr(1);
		// valueを取得
		strValue = L"";
		strDest = trim(strDest);
		strPosChar = strDest.substr(0, 1);
		if (strQuot.compare(strPosChar) != 0)
		{
			// quot記号が見つからなかった。
			std::wostringstream os;
			std::wstring wstrMessage;
			os << L"cfp_logger system error - quot記号があるべき場所に見つかりません。(";
			os << L"File:" << strFileName << L" Line " << s_numberOfLine;
			os << L":)\n";
			wstrMessage = os.str();
			::fwprintf(stderr, L"%s", wstrMessage.c_str());
			return vStringDummy;
		}
		strValue += strPosChar;
		strDest = strDest.substr(1);
		boValidValue = false;	// false値にて初期化
		while (0 < strDest.length())
		{
			strPosChar = strDest.substr(0, 1);
			if (2 <= strDest.length())
			{
				strPosWChar = strDest.substr(0, 2);
				if (strQuotQuot.compare(strPosWChar) == 0)
				{
					strDest = strDest.substr(2);
					strValue += strQuot;
					continue;
				}
			}
			strDest = strDest.substr(1);
			strValue += strPosChar;
			if (strQuot.compare(strPosChar) == 0)
			{
				boValidValue = true;
				break;
			}
		}
		if (boValidValue == false)
		{
			// valueが正しく閉じられていない。
			std::wostringstream os;
			std::wstring wstrMessage;
			os << L"cfp_logger system error - valueが正しく閉じられていません。(";
			os << L"File:" << strFileName << L" Line " << s_numberOfLine;
			os << L":)\n";
			wstrMessage = os.str();
			::fwprintf(stderr, L"%s", wstrMessage.c_str());
			return vStringDummy;
		}
		// 後コメント
		strDest = trim(strDest);
		if (strDest.empty())
		{
			// 後には何も続きません。
			break;
		}
		strPosChar = strDest.substr(0, 1);
		if (strSharp.compare(strPosChar) == 0)
		{
			// 後にコメントがある場合。
			break;
		}
		// 後に、#以外にて始まる文字列が存在する。
		std::wostringstream os;
		std::wstring wstrMessage;
		os << L"cfp_logger system error - 後に、sharp以外にて始まる文字列が存在する。(";
		os << L"File:" << strFileName << L" Line " << s_numberOfLine;
		os << L":)\n";
		wstrMessage = os.str();
		::fwprintf(stderr, L"%s", wstrMessage.c_str());
		return vStringDummy;
	} while (0);
	if ((1 <= strKey.length()) && (3 <= strValue.length()))
	{
		vString.push_back(strKey);
		vString.push_back(strValue.substr(1, strValue.length() - 2));
	}
	
	return vString;
}

/**
 * trim。
 * 
 * 文字列の前後よりスペースとタブを除去する。
 * 
 * @param[in] strSrc ソース文字列。
 * @return 前後のスペースとタブを除去された文字列。
 */
std::wstring trim(std::wstring strSrc)
{
	int iFirstPos, iLastPos, iPos;
	std::wstring strDelim = L" ";
	std::wstring strDelim2 = L"	";
	std::wstring strPosChar = L"";
	
	if (strSrc.empty())
	{
		return strSrc;
	}
	
	iFirstPos = -1;
	for (iPos = 0; iPos < (int)strSrc.length(); iPos++)
	{
		strPosChar = strSrc.substr(iPos, 1);
		if (strDelim.compare(strPosChar) != 0
				&& strDelim2.compare(strPosChar) != 0)
		{
			// スペースとタブ以外の文字が見つかった。
			iFirstPos = iPos;
			break;
		}
	}
	iLastPos = -1;
	for (iPos = (strSrc.length() - 1); 0 <= iPos; iPos--)
	{
		strPosChar = strSrc.substr(iPos, 1);
		if (strDelim.compare(strPosChar) != 0
				&& strDelim2.compare(strPosChar) != 0)
		{
			// スペースとタブ以外の文字が見つかった。
			iLastPos = iPos;
			break;
		}
	}
	
	if (iLastPos <= iFirstPos)
	{
		return L"";
	}
	
	return strSrc.substr(iFirstPos, iLastPos - iFirstPos + 1);
}
