// file:CLoggerStringUtils.cxx
/** @file
 * @brief 文字列・ユーティル・クラス・cxxファイル。
 * 
 * 文字列・ユーティルの実装。
 * @author yanch
 * @date 2006-10-21 作成
 * @date 2006-11-19 propファイルのパラメタ区切りにスペースが無い場合に対応
 * @date 2006-11-19 propファイルのパラメタ区切り文字にタブを使える様に対応
 * @date 2007-11-16 namespace に対応
 * @date 2007-11-16 unicode に対応
 * @date 2007-11-16 ソースファイルの拡張子をgccにならい、「.cxx」とした。
 * @date 2007-11-16 ヘッダファイルの拡張子を「.hxx」とした。
 * $Revision: 1.004$
 */
#define STRICT
#include <cstring>
#include <cwchar>
#include <cfp/logger/sub/CLoggerStringUtils.hxx>

/**
 * 文字コードをUTF-8よりUTF16へと変換。
 * 
 * @param[out] dest 出力文字列UTF-16
 * @param[in]  dest_size destのサイズをワード単位で指定
 * @param[in]  src 入力文字列UTF-8
 * @param[in]  src_size 入力文字列のバイト数
 * 
 * @return 成功時には出力文字列の文字数を戻します。
 *         dest_size に0を指定し、こちらの関数を呼び出すと、変換された
 *         文字列を格納するのに必要なdestのサイズの文字数を戻します。
 *         関数が失敗した場合には、falseを戻します。
 */
int cfp::logger::sub::CLoggerStringUtils::convertUtf8ToUtf16(
		wchar_t *dest, size_t dest_size, char *src, size_t src_size)
{
    const int nMaxReadSize = 6;
    size_t    countNeedsWords;
    size_t    cursor;
    char      chBuffer[nMaxReadSize];
    size_t    nReadDataSize;
    int       iCh1;
    int       sizeBytes;
    wchar_t   wcWork1, wcWork2, wcWork3;
    
    /*
     * 入力パラメータをチェック
     */
    if (dest_size == 0)
    {
        /* dest_size == 0 */
    }
    else
    {
        /* dest_size != 0 */
        if (dest == NULL)
        {
            /* Error -- Null Pointer Exception : dest */
            return false;
        }
        if (dest_size < 0)
        {
            /* Error -- dest_size < 0 */
            return false;
        }
    }
    if (src == NULL)
    {
        /* Error -- Null Pointer Exception : src */
        return false;
    }
    if (src_size < 1)
    {
        /* Error -- src_size < 1 */
        return false;
    }
    
    /*
     * BOMの除去
     */
    if ((*src == '\xef') && (*(src + 1) == '\xbb') && (*(src + 2) == '\xbf'))
    {
        /*
         * UTF-8のBOMはef bb bf
         * LE も BE も存在しないので、BOM付きの場合には除去。
         * BOMが無い場合はそのまま処理。
         */
        /* BOMがある場合にはBOMを無視する */
        src += 3;
    }
    
    countNeedsWords = 0;
    for (cursor = 0; cursor < src_size;)
    {
        /* srcより6バイトのデータを読み出し */
        nReadDataSize = (nMaxReadSize < (int)(src_size - cursor))?(nMaxReadSize):(src_size - cursor);
        memcpy(chBuffer, (src + cursor), nReadDataSize);
        memset(chBuffer + nReadDataSize, 0, sizeof(chBuffer) - nReadDataSize);

        /* data size の調べる */
        iCh1 = ((int)(*chBuffer)) & 0x00ff;
        iCh1 = ~iCh1;   /* ビット反転 */
        if (iCh1 & 0x0080)
        {
            /* 0aaabbbb */
            sizeBytes = 1;
        }
        else if (iCh1 & 0x0040)
        {
            /* error(ここに出現してはいけないコード) */
            return false;
        }
        else if (iCh1 & 0x0020)
        {
            /* 110aaabb 10bbcccc */
            sizeBytes = 2;
        }
        else if (iCh1 & 0x0010)
        {
            /* 1110aaaa 10bbbbcc 10ccdddd */
            sizeBytes = 3;
        }
        else if (iCh1 & 0x0008)
        {
            /* 未対応のマッピング(UTF-16に存在しないコード) */
            sizeBytes = 4;
        }
        else if (iCh1 & 0x0004)
        {
            /* 未対応のマッピング(UTF-16に存在しないコード) */
            sizeBytes = 5;
        }
        else if (iCh1 & 0x0002)
        {
            /* 未対応のマッピング(UTF-16に存在しないコード) */
            sizeBytes = 6;
        }
        else
        {
            /* error(ここに出現してはいけないコード) */
            return false;
        }
        
        /*
         * dest_size をチェック
         */
        if (dest_size && (dest_size < (countNeedsWords + 1)))
        {
            /* Error : memory is not enough for dest */
            return countNeedsWords;
        }
        
        /* sizeBytes毎に処理を分岐 */
        if (dest_size) switch (sizeBytes)
        {
        case 1:
            /*
             * ビット列
             * (0aaabbbb)UTF-8 ... (00000000 0aaabbbb)UTF-16
             */
            *dest = ((wchar_t)(chBuffer[0])) & (wchar_t)0x00ff;   /* 00000000 0aaabbbb */
            dest++;
            break;
        case 2:
            /*
             * ビット列
             * (110aaabb 10bbcccc)UTF-8 ... (00000aaa bbbbcccc)UTF-16
             */
            wcWork1 = ((wchar_t)(chBuffer[0])) & (wchar_t)0x00ff; /* 00000000 110aaabb */
            wcWork2 = ((wchar_t)(chBuffer[1])) & (wchar_t)0x00ff; /* 00000000 10bbcccc */
            wcWork1 <<= 6;                                          /* 00110aaa bb?????? */
            wcWork1 &= 0x07c0;                                      /* 00000aaa bb000000 */
            wcWork2 &= 0x003f;                                      /* 00000000 00bbcccc */
            *dest = wcWork1 | wcWork2;                              /* 00000aaa bbbbcccc */
            dest++;
            break;
        case 3:
            /*
             * ビット列
             * (1110aaaa 10bbbbcc 10ccdddd)UTF-8 ... (aaaabbbb ccccdddd)UTF-16
             */
            wcWork1 = ((wchar_t)(chBuffer[0])) & (wchar_t)0x00ff; /* 00000000 1110aaaa */
            wcWork2 = ((wchar_t)(chBuffer[1])) & (wchar_t)0x00ff; /* 00000000 10bbbbcc */
            wcWork3 = ((wchar_t)(chBuffer[2])) & (wchar_t)0x00ff; /* 00000000 10ccdddd */
            wcWork1 <<= 12;                                         /* aaaa???? ???????? */
            wcWork1 &= 0xf000;                                      /* aaaa0000 00000000 */
            wcWork2 <<= 6;                                          /* 0010bbbb cc?????? */
            wcWork2 &= 0x0fc0;                                      /* 0000bbbb cc000000 */
            wcWork3 &= 0x003f;                                      /* 00000000 00ccdddd */
            *dest = wcWork1 | wcWork2 | wcWork3;                    /* aaaabbbb ccccdddd */
            dest++;
            break;
        case 4:
        case 5:
        case 6:
        default:
            /* ダミーデータ(uff1f)を出力 */
            *dest = (wchar_t)0xff1f;
            dest++;
            break;
        }
        countNeedsWords++;
        cursor += sizeBytes;
    }
    
    return countNeedsWords;
}

/**
 * 文字コードをUTF-16よりUTF-8へと変換。
 * 
 * @param[out] dest 出力文字列UTF-8
 * @param[in]  dest_size destのバイト数
 * @param[in]  src 入力文字列UTF-16
 * @param[in]  src_size 入力文字列の文字数
 * 
 * @return 成功時には出力文字列のバイト数を戻します。
 *         dest_size に0を指定し、こちらの関数を呼び出すと、変換された
 *         文字列を格納するのに必要なdestのバイト数を戻します。
 *         関数が失敗した場合には、falseを戻します。
 */
int cfp::logger::sub::CLoggerStringUtils::convertUtf16ToUtf8(
		char *dest, size_t dest_size, wchar_t *src, size_t src_size)
{
    long        countNeedsBytes;
    long        cursor;
    wchar_t     wcWork1;
    long        sizeBytes;
    char       chWork1;
    char       chWork2;
    char       chWork3;
    
    /*
     * 入力パラメータをチェック
     */
    if (dest_size == 0)
    {
        /*
         * dest_size == 0
         */
    }
    else
    {
        /*
         * dest_size != 0
         */
        if (dest == NULL)
        {
            /* Error : dest is NULL. */
            return false;
        }
        if (dest_size < 0)
        {
            /* Error : dest_size < 0. */
            return false;
        }
    }
    if (src == NULL)
    {
        /* Error : src is NULL. */
        return false;
    }
    if (src_size < 0)
    {
        /* Error : src_size < 0. */
        return false;
    }
    
    countNeedsBytes = 0;
    for (cursor = 0; cursor < (long)src_size; cursor++)
    {
        /* srcより1ワードのデータを読み出し */
        wcWork1 = *(src + cursor);
        if (       (wcWork1 <= ((wchar_t)0x007f)))
        {
            /* 0x0000 to 0x007f */
            sizeBytes = 1;
        }
        else if (  (((wchar_t)0x0080) <= wcWork1)
                && (wcWork1 <= ((wchar_t)0x07ff)))
        {
            /* 0x0080 to 0x07ff */
            sizeBytes = 2;
        }
        else if (  (((wchar_t)0x0800) <= wcWork1))
        {
            /* 0x0800 to 0xffff */
            sizeBytes = 3;
        }
        else
        {
            /* Error : unknown code */
            return false;
        }
        
        /*
         * dest_size をチェック
         */
        if (dest_size && ((long)dest_size < (countNeedsBytes + sizeBytes)))
        {
            /* Error : memory is not enough for dest */
            return countNeedsBytes;
        }
        
        /* sizeBytes毎に処理を分岐 */
        if (dest_size) switch (sizeBytes)
        {
        case 1:
            /*
             * ビット列
             * (0aaabbbb)UTF-8 ... (00000000 0aaabbbb)UTF-16
             */
            *dest = (char)wcWork1;              /* 0aaabbbb */
            dest ++;
            break;
        case 2:
            /*
             * ビット列
             * (110aaabb 10bbcccc)UTF-8 ... (00000aaa bbbbcccc)UTF-16
             */
            chWork1 = (char)(wcWork1 >> 6);     /* 000aaabb */
            chWork1 |= (char)0xc0;              /* 110aaabb */
            chWork2 = (char)wcWork1;            /* bbbbcccc */
            chWork2 &= (char)0x3f;              /* 00bbcccc */
            chWork2 |= (char)0x80;              /* 10bbcccc */
            *dest = chWork1;
            dest++;
            *dest = chWork2;
            dest++;
            break;
        case 3:
            /*
             * ビット列
             * (1110aaaa 10bbbbcc 10ccdddd)UTF-8 ... (aaaabbbb ccccdddd)UTF-16
             */
            chWork1 = (char)(wcWork1 >> 12);    /* ????aaaa */
            chWork1 &= (char)0x0f;              /* 0000aaaa */
            chWork1 |= (char)0xe0;              /* 1110aaaa */
            chWork2 = (char)(wcWork1 >> 6);     /* aabbbbcc */
            chWork2 &= (char)0x3f;              /* 00bbbbcc */
            chWork2 |= (char)0x80;              /* 10bbbbcc */
            chWork3 = (char)wcWork1;            /* ccccdddd */
            chWork3 &= (char)0x3f;              /* 00ccdddd */
            chWork3 |= (char)0x80;              /* 10ccdddd */
            *dest = chWork1;
            dest++;
            *dest = chWork2;
            dest++;
            *dest = chWork3;
            dest++;
            break;
        default:
            break;
        }
        countNeedsBytes += sizeBytes;
    }
    
    return countNeedsBytes;
}
